#include "bintree2.h"

#include <iostream>

using namespace std;

namespace bintree {

struct Tree::Node
{
    string key;
    unsigned value;
    link_t left;
    link_t right;
};

size_t size(const Tree& tree) {
    return tree.size;
}

// Finds the subtree associated with a particular key.
Tree::link_t find_subtree(Tree::link_t, const std::string& key);
Tree::link_t& find_subtree_mut(Tree::link_t&, const std::string& key);

bool member(const Tree& tree, const string& key)
{
    return find_subtree(tree.root, key) != nullptr;
}

unsigned lookup(const Tree& tree, const string& key)
{
    Tree::link_t node = find_subtree(tree.root, key);
    if (node == nullptr)
        throw Not_found(key);
    else return node->value;
}

void insert(Tree& tree, const string& key, unsigned value)
{
    Tree::link_t& node = find_subtree_mut(tree.root, key);
    if (node == nullptr) {
        node = make_shared<Tree::Node>();
        node->key = key;
        ++tree.size;
    }
    node->value = value;
}

Tree::link_t find_subtree(Tree::link_t link, const string& key)
{
    while (link != nullptr) {
        if (key < link->key) link = link->left;
        else if (key > link->key) link = link->right;
        else return link;
    }

    return nullptr;
}

Tree::link_t& find_subtree_mut(Tree::link_t& link, const string& key)
{
    if (link == nullptr) return link;
    if (key < link->key) return find_subtree_mut(link->left, key);
    if (key > link->key) return find_subtree_mut(link->right, key);
    return link;
}

Tree::link_t& find_subtree_mut_iter(Tree::link_t& link, const string& key)
{
    if (link == nullptr || link->key == key)
        return link;

    Tree::link_t curr = link;
    for (;;) {
        if (key < curr->key) {
            if (curr->left == nullptr || key == curr->left->key)
                return curr->left;
            else
                curr = curr->left;
        } else {
            if (curr->right == nullptr || key == curr->right->key)
                return curr->right;
            else
                curr = curr->right;
        }
    }
}

void get_keys_helper(Tree::link_t node, vector<string>& out)
{
    if (node != nullptr) {
        get_keys_helper(node->left, out);
        out.push_back(node->key);
        get_keys_helper(node->right, out);
    }
}

vector<string> get_keys(const Tree& tree)
{
    vector<string> result;
    get_keys_helper(tree.root, result);
    return result;
}

}
