#include "bintree4.h"
#include <catch.h>

using namespace bintree;

TEST_CASE("Member_empty")
{
    Tree tree;
    CHECK_FALSE(tree.member("a"));
}

TEST_CASE("Lookup_insert_throws")
{
    Tree tree;
    CHECK_THROWS_AS(tree.lookup("a"), Not_found);
}

TEST_CASE("Insert_first_element_then_lookup")
{
    Tree tree;
    tree.insert("b", 5);

    CHECK_FALSE(tree.member("a"));
    CHECK(tree.member("b"));
    CHECK(5 == tree.lookup("b"));
}

TEST_CASE("Insert_several")
{
    Tree tree;
    tree.insert("d", 4);
    tree.insert("b", 2);
    tree.insert("a", 1);

    CHECK(tree.member("a"));
    CHECK(tree.member("b"));
    CHECK_FALSE(tree.member("c"));
    CHECK(tree.member("d"));
    CHECK_FALSE(tree.member("e"));
    CHECK_FALSE(tree.member("f"));
    CHECK_FALSE(tree.member("g"));

    tree.insert("c", 3);
    tree.insert("f", 6);
    tree.insert("e", 5);
    tree.insert("g", 7);

    CHECK(tree.member("a"));
    CHECK(tree.member("b"));
    CHECK(tree.member("c"));
    CHECK(tree.member("d"));
    CHECK(tree.member("e"));
    CHECK(tree.member("f"));
    CHECK(tree.member("g"));
    CHECK_FALSE(tree.member(""));
    CHECK_FALSE(tree.member("ax"));
    CHECK_FALSE(tree.member("bx"));
    CHECK_FALSE(tree.member("cx"));
    CHECK_FALSE(tree.member("dx"));
    CHECK_FALSE(tree.member("ex"));
    CHECK_FALSE(tree.member("fx"));
    CHECK_FALSE(tree.member("gx"));

    CHECK(1 == tree.lookup("a"));
    CHECK(2 == tree.lookup("b"));
    CHECK(3 == tree.lookup("c"));
    CHECK(4 == tree.lookup("d"));
    CHECK(5 == tree.lookup("e"));
    CHECK(6 == tree.lookup("f"));
    CHECK(7 == tree.lookup("g"));

    CHECK_THROWS_AS(tree.lookup("h"), Not_found);
}

